#!/usr/bin/perl

use File::Copy;
use File::Basename;
use File::Spec;
use File::Path;

my $return_code = 0;
my $devnul = "";
my $interactive = 0;

$SERVER_ROOT = $ARGV[0];
$CUR_DIR = `chdir`; chomp $CUR_DIR;

sub change_dir {
  my $dir = shift;
  print "Change dir to $dir\n";
  return chdir $dir;
}

sub cmd {
  $c = shift;
  print "$c\n";
  system("$c");
  if( $? ) {
    print "Error: $!, exiting\n";
    exit 1;
  }
}

sub cmd_noerr {
  $c = shift;
  print "$c\n";
  system("$c");
  if( $? ) {
    return 1;
  }
  return 0;
}

sub cp {
  my $c1 = shift; my $c2 = shift;
  print "cp $c1 $c2\n";
  my $res = copy $c1, $c2;
  if( ! $res ) {
    print "Error, $!\n";
  }
  return $res;
}

if ( $#ARGV == 0 ) {
  $interactive=1;
}

if ( ! -d $SERVER_ROOT ) {
  print "$SERVER_ROOT directory does not exist\n";
  exit 1;
}

my $CREDS_FILE,$cred_option;

if (! $interactive) {

   $log_file = File::Spec->catfile("$SERVER_ROOT","sync-log");
   
   open(LOG, ">$log_file") or die "Can't create logfile: $!";
   close LOG; 
   $devnul = " >> \"$log_file\" 2>&1 ";

   $CREDS_FILE = File::Spec->catfile("$CUR_DIR", "creds");
   
   if ( "$ARGV[1]" eq "-f" ) {
     if ( ! -f "$ARGV[2]" ) {
        print "Error: $ARGV[2] does not exist\n";
        exit 1;
     }
    cp "$ARGV[2]" , $CREDS_FILE;
   }
   else
   {
    open(CREDS, ">$CREDS_FILE") || die "Could not create : creds, $!";
    print CREDS "Admin Id: $ARGV[1]\n";
    print CREDS "Admin Password: $ARGV[2]\n";
    close CREDS;
   }
 $cred_option=" -f \"$CREDS_FILE\" ";
} else {
 $cred_option="";
}

# Keep trace of presence of a slap- directory
# (which means there's at least one DS installed in 
# the server root.
my $DS_installed = 0;

opendir(SERVERDIR, "$SERVER_ROOT") || die "Could not open directory : $SERVER_ROOT, $!";
@instance_dirs = grep { /^slapd-/ } readdir SERVERDIR;
for (@instance_dirs) {
  my $stop_slapd_dir = File::Spec->catfile("$SERVER_ROOT", $_);
  if( -d $stop_slapd_dir) {
    $DS_installed = 1;
  }
}

if($DS_installed) {
  for (@instance_dirs) {
    my $start_dir = File::Spec->catfile("$SERVER_ROOT","$_");
    if( -d $start_dir) {
      print "cd $start_dir\n";
      change_dir $start_dir;
      my $start_slapd = File::Spec->catfile(File::Spec->curdir(), "start-slapd");
      print "Starting instance : $_\n";
      cmd "$start_slapd";
    }
  }
  
  print "Please wait 20 seconds...\n";
  sleep(20);
}

my $new_dir = File::Spec->catdir("$SERVER_ROOT", "bin", "admin");
change_dir $new_dir;

my $sync_admin = File::Spec->catfile(File::Spec->curdir(), "sync-admin");
$sync_admin = $sync_admin . " upgrade -r \"$SERVER_ROOT\"".$devnul;
print "Upgrading the Administration Server...\n";
my $res = cmd_noerr $sync_admin;
if( $res ) {
  print "Warning, sync-admin upgrade returned with error, the upgrade might have already been done ?\n";
  $return_code += 1;
}

my $sync_admin_cds = File::Spec->catfile(File::Spec->curdir(), "sync-admin-cds");
$sync_admin_cds_cmd = $sync_admin_cds . " -r \"$SERVER_ROOT\"".$cred_option.$devnul;
print "Updating the Configuration Directory Server for the Admin Server...\n";
$res = cmd_noerr $sync_admin_cds_cmd;
if( $res ) {
  print "Warning, sync-admin-cds returned with error\n";
  print "Verify user and password are the right ones and that CDS is up and running\n";
  print "You could run the following commands once upgrade is finished:\n";
  print "--------------------------\n";
  print "cd $new_dir\n";
  print "$sync_admin_cds -r \"$SERVER_ROOT\"\n";
  print "--------------------------\n";
  $return_code += 1;
}

# And start the admin server

print "Starting the admin server...\n";
change_dir $SERVER_ROOT;
my $start_admin = File::Spec->catfile(File::Spec->curdir(), "start-admin");
$return_code += cmd_noerr "$start_admin";

if (! $interactive) {
 unlink "$CREDS_FILE";
}

print "\n\nDone\n\n";

if ($return_code == 0) {
  unlink "$log_file";
}

exit $return_code;